library Comm;

uses
  SysUtils,
  Classes,
  Controls,
  Rubies,
  Pythia,
  CommX,
  CommHandle;

var
  Handle: TCommHandle;

function Comm_new(argc: Integer; argv: Pointer; this: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  if argc <> 0 then ap_raise(ap_eArgError, sWrong_num_of_args);
  real := TCommX.Create(nil);
  result := CompoAlloc(this, real);

  if @real.OnReceive = nil then real.OnReceive := handle.doReceive;
  if @real.OnBreak = nil then real.OnBreak := handle.doBreak;
  if @real.OnError = nil then real.OnError := handle.doError;

  rb_funcall2(result, id_init, argc, argv);
end;

function Comm_send_break(this: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  real := ap_data_get_struct(this);
  real.SendBreak;
  result := Qnil;
end;

function Comm_open(this: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  real := ap_data_get_struct(this);
  try
    real.PortOpen;
  except
    on E: ECommError do ap_raise(ap_eIOError, E.message);
  end;
  result := Qnil;
end;

function Comm_close(this: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  real := ap_data_get_struct(this);
  try
    real.PortClose;
  except
    on E: ECommError do ap_raise(ap_eIOError, E.message);
  end;
  result := Qnil;
end;

function Comm_clear_receive_buf(this: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  real := ap_data_get_struct(this);
  real.ClearReceiveBuf;
  result := Qnil;
end;

function Comm_clear_send_buf(this: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  real := ap_data_get_struct(this);
  real.ClearTransBuf;
  result := Qnil;
end;

function Comm_send_char(this, c: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  real := ap_data_get_struct(this);
  result := ap_bool(real.TransChar(NUM2CHR(c)));
end;

function Comm_receive_char(this: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  real := ap_data_get_struct(this);
  result := INT2FIX(real.ReceiveChar);
end;

function Comm_receive(this, str: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  Check_Type(str, T_STRING);
  real := ap_data_get_struct(this);
  result := INT2FIX(real.ReceiveBlock(ap_str_ptr(str), ap_str_len(str)));
end;

function Comm_send(this, str: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  Check_Type(str, T_STRING);
  real := ap_data_get_struct(this);
  result := ap_bool(real.TransBlock(ap_str_ptr(str)^, ap_str_len(str)));
end;

function Comm_get_send_length(this: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  real := ap_data_get_struct(this);
  result := INT2FIX(real.GetTransLength);
end;

function Comm_get_receive_length(this: Tvalue): Tvalue; cdecl;
var
  real: TCommX;
begin
  real := ap_data_get_struct(this);
  result := INT2FIX(real.GetReceiveLength);
end;

procedure Init_comm; cdecl;
begin
  id_init := rb_intern('initialize');

  Handle := TCommHandle.Create;
  PhiExtentList.Add(Handle);

  rb_eCommError := rb_define_class('CommError', ap_eStandardError);

  cComm := rb_define_class('Comm', ap_cControl);
  DefineProp(cComm, TCommX);
  DefineConstSetType(cComm, TypeInfo(TCommParityBits));
  DefineConstSetType(cComm, TypeInfo(TCommStopBits));
  DefineConstSetType(cComm, TypeInfo(TCommFlowCtrls));

  rb_define_method(cComm, 'send_break', @Comm_send_break, 0);
  rb_define_method(cComm, 'open', @Comm_open, 0);
  rb_define_method(cComm, 'close', @Comm_close, 0);
  rb_define_method(cComm, 'clear_receive_buf', @Comm_clear_receive_buf, 0);
  rb_define_method(cComm, 'clear_send_buf', @Comm_clear_send_buf, 0);

  DefineAttrGet(cComm, 'send_length', Comm_get_send_length);
  DefineAttrGet(cComm, 'receive_length', Comm_get_receive_length);

  rb_define_method(cComm, 'send_char', @Comm_send_char, 1);
  rb_define_method(cComm, 'send', @Comm_send, 1);
  rb_define_method(cComm, 'receive_char', @Comm_receive_char, 0);
  rb_define_method(cComm, 'receive', @Comm_receive, 1);

  DefineSingletonMethod(cComm, 'new', Comm_new);

  rb_iv_set(cComm, '@events', rb_hash_new);

end;

exports
  Init_comm;

begin

end.

